package SOF2Logger::ELORating;

# Simple package that uses the ELO rating system to update 2 players
# ratings after a kill.

use strict;

use constant MAX_RATING_INCREASE => 32;

use base qw/Exporter/;

our @EXPORT_OK = qw/update_rating/;


# update rating takes a killer player and a victim player, and updates
# their ratings according to the ELO algorithm
sub update_rating
{
    my ($killer, $victim) = @_;

    # in case we don't have both players, which should never happen,
    # but its certainly possible with malformed log files.
    return unless $killer && $victim;

    my %E;

    $E{$killer} = ELO_expected_score($victim->rating, $killer->rating);
    $E{$victim} = ELO_expected_score($killer->rating, $victim->rating);


    # R' = R + K * (S - E)
    $killer->rating( sprintf "%.0f", $killer->rating + MAX_RATING_INCREASE * ( 1 - $E{$killer} ) );
    $victim->rating( sprintf "%.0f", $victim->rating + MAX_RATING_INCREASE * ( 0 - $E{$victim} ) );



}

sub ELO_expected_score
{
    my ($r1, $r2) = @_;

    # E(B) = 1 / [ 1 + 10 ^ ( [R(A) - R(B)] / 400 ) ]

    return 1 / ( 1 + 10 ** ( ($r1 - $r2) / 400 ) );
}

1;
