package SOF2Logger::OutputHTML;

use strict;

use base qw/Exporter/;

use SOF2Logger::Utils qw/filename_for_player/;

our @EXPORT_OK = qw/
  html_authors_links
  html_homepage_link
  popular_weapon
  worst_enemy_header
  worst_enemy_for_player
  worst_enemy_footer
  player_stats_header
  most_popular_targets_header
  most_popular_targets_for_player
  most_popular_targets_footer
  most_popular_weapons_header
  most_popular_weapons_for_player
  most_popular_weapons_footer
  kills_by_weapon_players
  main_stats_footer
  main_stats_for_player
  main_stats_header

  convert_seconds_to_time_format
  htmlize_and_link_player
  /;

=head1 NAME

OutputHTML.pm : the output package

=head1 DESCRIPTION

This package stores the subs that turn the raw stats into HTML-formatted 
beauty.

=head2 PUBLIC INTERFACE

=over 8

=cut

# global semi-constant var
our $image_dir;
my $cell_num;

my %COLOR_MAP = (
                  '0'  => "#000000",    # BLACK
                  '1'  => "#FF0000",    # RED
                  '2'  => "#80FF00",    # GREEN
                  '3'  => "#FFFF00",    # YELLOW
                  '4'  => "#0000FF",    # BLUE
                  '5'  => "#00FFFF",    # CYAN
                  '6'  => "#FF00FF",    # MAGENTA
                  '7'  => "#FFFFFF",    # WHITE
                  '8'  => "#AD19D5",    # PURPLE
                  '9'  => "#A7F4F1",    # CYAN
                  'a'  => "#48E2DC",    # CYAN
                  'b'  => "#FAE3A9",    # ORANGE
                  'c'  => "#E5A7E5",    # PINK
                  'd'  => "#DEF7FE",    # BLUE
                  'e'  => "#D71BBC",    # PINK
                  'f'  => "#CFFAFB",    # BLUE
                  'g'  => "#D1F783",    # GREEN
                  'h'  => "#A91859",    # RED
                  'i'  => "#789BCB",    # BLUE
                  'j'  => "#9BEE44",    # GREEN
                  'k'  => "#FEFCDA",    # TAN
                  'l'  => "#EEFBFD",    # BLUE
                  'm'  => "#DB79DA",    # PINK
                  'n'  => "#ACC997",    # GREEN
                  'o'  => "#E9C9CB",    # PINK
                  'p'  => "#84D6C0",    # CYAN
                  'q'  => "#FAEEF1",    # TAN
                  'r'  => "#E5A7E5",    # PINK
                  's'  => "#6115A3",    # BLUE
                  't'  => "#B31164",    # RED
                  'u'  => "#846FD0",    # BLUE
                  'v'  => "#5BE593",    # GREEN
                  'w'  => "#000000",    # BLACK
                  'x'  => "#0C2C4C",    # BLUE
                  'y'  => "#48E2DC",    # BLUE
                  'z'  => "#EFD497",    # ORANGE
                  'A'  => "#7918CE",    # PURPLE
                  'B'  => "#9CED8B",    # GREEN
                  'C'  => "#E3AA87",    # ORANGE
                  'D'  => "#889CB8",    # BLUE
                  'E'  => "#CEEF99",    # GREEN
                  'F'  => "#1F5C9A",    # BLUE
                  'G'  => "#A8EB95",    # GREEN
                  'H'  => "#D5CCCC",    # PINK
                  'I'  => "#C0F0F0",    # BLUE
                  'J'  => "#F5FCEF",    # GRAY
                  'K'  => "#F6FEDC",    # GREEN
                  'L'  => "#F1DAF7",    # PURPLE
                  'M'  => "#44E0BA",    # CYAN
                  'N'  => "#EBDA6F",    # ORANGE
                  'O'  => "#FFFFFF",    # WHITE
                  'P'  => "#35D936",    # GREEN
                  'Q'  => "#F79DF1",    # PINK
                  'R'  => "#7686D7",    # BLUE
                  'S'  => "#9CED8B",    # GREEN
                  'T'  => "#DDAB9F",    # ORANGE
                  'U'  => "#CDA3E5",    # PURPLE
                  'V'  => "#DBFAF8",    # GRAY
                  'W'  => "#9F25B8",    # PURPLE
                  'X'  => "#6FF2EE",    # CYAN
                  'Y'  => "#D4F353",    # GREEN
                  'Z'  => "#F8EEFD",    # PURPLE
                  '!'  => "#000000",    # BLACK
                  '@'  => "#F7D43B",    # ORANGE
                  '#'  => "#000000",    # BLACK
                  '$'  => "#C71819",    # RED
                  '&'  => "#A39DC5",    # BLUE
                  '*'  => "#F7FCFB",    # WHITE
                  '('  => "#CD4624",    # PINK
                  ')'  => "#40E3E7",    # CYAN
                  '_'  => "#A82526",    # RED
                  '-'  => "#8A8789",    # GRAY
                  '+'  => "#FFFFFF",    # WHITE
                  '='  => "#BED4C9",    # CYAN
                  '{'  => "#CD4624",    # RED
                  '['  => "#A5EEE1",    # CYAN
                  '}'  => "#6E939B",    # BLUE
                  ']'  => "#89EEE4",    # CYAN
                  '|'  => "#D6F7B5",    # GREEN
                  '\\' => "#D6ABCE",    # PINK
                  ':'  => "#FFFFFF",    # WHITE
                  '\'' => "#8A8789",    # GRAY
                  '<'  => "#2CE02F",    # GREEN
                  ','  => "#DDE8D6",    # GRAY
                  '>'  => "#81F2F1",    # CYAN
                  '.'  => "#A39DC5",    # BLUE
                  '?'  => "#D6F7B5",    # GREEN
                  '/'  => "#411959"     # PURPLE'
);

# MAIN PAGE STAT INFO

=item main_stats_header CONFIG

=cut

sub main_stats_header ($) {
  my ($config)  = @_;
  my $stattype  = shift;
  my $top_num   = $config->top_num;
  my $game_type = $config->game_type;
  my $mod = $config->game_mod || '';

  return qq(
<table class="sortable dataset" id="main_stats_table">
  <caption>Top $top_num $game_type $mod players on this server</caption>
  <thead>
    <tr>
      <th class="header" colspan="2"><a href="" onclick="this.blur(); return sortTable('mainStatsForPlayer', 1, false);" title="Player Name">Player</a></th>
      <th class="header"><a href="" onclick="this.blur(); return sortTable('mainStatsForPlayer', 2, true);" title="Kills">Kills</a></th>
      <th class="header"><a href="" onclick="this.blur(); return sortTable('mainStatsForPlayer', 3, true);" title="Deaths">Dths</a></th>
      <th class="header"><a href="" onclick="this.blur(); return sortTable('mainStatsForPlayer', 4, true);" title="Suicides">Suic</a></th>
      <th class="header"><a href="" onclick="this.blur(); return sortTable('mainStatsForPlayer', 5, true);" title="Flag Captures">Caps</a></th>
      <th class="header"><a href="" onclick="this.blur(); return sortTable('mainStatsForPlayer', 6, true);" title="Flag Returns">Rets</a></th>
      <th class="header"><a href="" onclick="this.blur(); return sortTable('mainStatsForPlayer', 7, true);" title="Play Time">Time</a></th>
      <th class="header"><a href="" onclick="this.blur(); return sortTable('mainStatsForPlayer', 8, true);" title="Kills per Hour">KPH</a></th>
      <th class="header"><a href="" onclick="this.blur(); return sortTable('mainStatsForPlayer', 9, true);" title="Player Rating">Rating</a></th>
      <th class="header"><a href="" onclick="this.blur(); return sortTable('mainStatsForPlayer', 10, true);" title="Most Efficient">Eff</a></th>
    </tr>
  </thead>
  <tbody id="mainStatsForPlayer">
    );
}

#       <th class="header"><a href="#" class="sortheader" title="Player">Player</a></th>
#       <th class="header"><a href="#" class="sortheader" title="Kills">Kills</a></th>
#       <th class="header"><a href="#" class="sortheader" title="Deaths">Dths</a></th>
#       <th class="header"><a href="#" class="sortheader" title="Suicides">Suic</a></th>
#       <th class="header"><a href="#" class="sortheader" title="Flag Captures">Caps</a></th>
#       <th class="header"><a href="#" class="sortheader" title="Flag Returns">Rets</a></th>
#       <th class="header"><a href="#" class="sortheader" title="Play Time">Time</a></th>
#       <th class="header"><a href="#" class="sortheader" title="Kills per Hour">KPH</a></th>
#       <th class="header"><a href="#" class="sortheader" title="Rating">Rating</a></th>
#       <th class="header"><a href="#" class="sortheader" title="Efficiency">Eff</a></th>




sub main_stats_for_player {
  my (
       $count,  $player,         $output_dir, $kills,
       $deaths, $suicides,       $captures,   $returns,
       $time,   $kills_per_hour, $rating,     $bar_length
    )
    = @_;

  my $html = stat_line_with_bar(
                                "$count.",
                                htmlize_and_link_player( $player, $output_dir ),
                                $kills,
                                $deaths,
                                $suicides,
                                $captures,
                                $returns,
                                convert_seconds_to_time_format($time),
                                $kills_per_hour,
                                $rating,
                                [$bar_length]
  );

  return $html;
}

sub main_stats_footer {
  return '</tbody></table>';
}

=item popular_weapon $%WEAPON TOTAL_KILLS $%WEAPON_MAP

Most popular weapon for all players.  Displays on main page.

HTML string

=cut

sub popular_weapon {
  my ( $total_kills, $weapons_info ) = @_;
  my $pop_weapon;
  my $count = 1;

  my @rows;

  for my $info (@$weapons_info) {
    my ( $weapon, $kills ) = @$info;
    my $bar_length = 100 * ( $kills / $total_kills );

    push @rows, [ "$count. $weapon", $kills, html_bar($bar_length) ];

    $count++;
  }

  return render_dataset( "Most Popular Weapons",
                      [ "Weapon", "Kills", "Usage Ratio", "Top User" ], @rows );
}

=item kills_by_weapon_players TOTAL_KILLS LIMIT PLAYERS
Players is an array of arrays that has the players_name and their kills for
WEAPON_NAME, sorted descending by number of kills

ie. [ ['sue', 24], ['billy', 3] ]

=cut

sub kills_by_weapon_players {
  my ( $total_kills, $limit, $weapon_name, $config, @players ) = @_;

  my $count = 1;
  my @rows;
  foreach my $player_info (@players) {
    my ( $player, $kills ) = @$player_info;

    last if $count > $limit;
    next if $player->is_filtered;

    my $bar_length = 100 * ( $kills / $total_kills );
    my $name = htmlize_and_link_player( $player, $config->player_data_dir );

    push @rows, [ "$count. $name", $kills, html_bar($bar_length) ];

    $count++;
  }

  return render_dataset(
                 "Top $limit (<strong>$weapon_name</strong>) - Weapon Handlers",
                 [qw/Players Kills Ratio/], @rows );
}

# PROGRAM INFO

=item html_homepage_link

takes a homepage link and turns it into its HTML version

=cut

sub html_homepage_link {
  my ($home_page) = @_;

  return "<a href=\"$home_page\">$home_page</a>";
}

=item html_authors links %$AUTHORS

takes a ref to a hash that has AUTHOR_NAME => EMAIL and converts them to
their html name and linked email address

=cut

sub html_authors_links {
  my ($authors) = @_;

  my $html_str = '';

  while ( my ( $name, $email ) = each %$authors ) {
    $html_str .= "<br>$name <a href=\"mailto:$email\">$email</a>\n";
  }

  return $html_str;
}

################################################################################
# PLAYER DETAIL METHODS
################################################################################

=item worst_enemy_header PLAYER_NAME

Note:Individual

=cut

sub worst_enemy_header {
  my ($player_name) = @_;

  $player_name = strip_colors($player_name);

  return qq(
    <table class="eghtpctpad">
      <tr>
        <th class="header" colspan="3">Worst Enemies for $player_name</th>
      </tr>
      <tr>
        <th class="header">Enemy</th>
        <th class="header">Deaths</th>
        <th class="header">Ratio of total Deaths</th>
      </tr>
    );

}

=item worst_enemy_for_player COUNT ENEMY DEATHS TOTAL_DEATHS DEPTH IS_ENEMY_RATED

Formats the worst enemy for a player as an .html table row
Note:Individual

=cut

sub worst_enemy_for_player {
  my ( $count, $enemy, $deaths, $total_deaths, $levels ) = @_;

  my $bar_length = ( 100 * $deaths ) / $total_deaths;
  my $enemy_html = htmlize_and_link_player($enemy);

  return stat_line_with_bar( "$count. $enemy_html (rating: @{[ $enemy->rating ]})",
                             $deaths, [ $bar_length, $levels ] );
}

=item worst_enemy_footer

Note:Individual

=cut

sub worst_enemy_footer {
  return '</table>';
}

sub player_stats_header {
  my (
       $config,     $name,       $kills,   $deaths,
       $suicides,   $captures,   $returns, $rating,
       $headshots,  $bar_length, $time,    $avatar
    )
    = @_;
  my $depth       = 1;
  my $avatar_html = $avatar
    ? sprintf( qq|<img style="float:left" src="../ig/%s">|, $avatar )
    : "";

  if ( !-e $config->output_dir . "/ig/$avatar" ) {
    warn "No avatar image file ($avatar) for player $name\n";
  }

  $time = convert_seconds_to_time_format($time);
  $name = strip_colors($name);

  my $html = qq(
    <table class="eghtpctpad">
      <tr class="cell1">
        <th class="header" colspan="9">$avatar_html<h1>${name}</h1></th>
      </tr>
      <tr class="cell1">
        <th class="header">Kills</th>
        <th class="header">Deaths</th>
        <th class="header">Suicides</th>
        <th class="header">Captures</th>
        <th class="header">Returns</th>
        <th class="header">Rating</th>);

  # no headshots for JK2,
  # JJJ i bet there's a better way to do this...
  if ( !( $config->game_type eq 'JK2' ) ) {
    $html .= qq(<th class="header">Headshots</th>);
  }

  $html .= qq(
        <th class="header">Time</th>
        <th class="header">Efficiency</th>
      </tr>
    );

  $html .= stat_line_with_bar(
                $kills,    $deaths,
                $suicides, $captures,
                $returns, $rating,
                ( $config->game_type eq 'JK2' ? () : $headshots ),
                $time, [ $bar_length, $depth ]
  );

  $html .= '</table>';

  return $html;
}

=item most_popular_targets_header PLAYER_NAME

The header for the most popular targets

=cut

sub most_popular_targets_header {
  my ($name) = @_;

  $name = strip_colors($name);

  return qq(
    <table class="eghtpctpad">
    <tr>
     <th class="header" colspan="3">Favorite Targets for ${name}</th>
    </tr>
    <tr>
     <th class="header">Victim</th>
     <th class="header">Kills</th>
     <th class="header">Ratio of total Kills</th>
    </tr>
    );
}

=item most_popular_targets_for_player CONFIG COUNT TARGET KILLS TOTAL_KILLS DEPTH

=cut

sub most_popular_targets_for_player {
  my ( $count, $target, $deaths, $total_kills, $levels ) = @_;

  my $bar_length  = ( 100 * $deaths ) / $total_kills;
  my $target_html = htmlize_and_link_player($target);

  return stat_line_with_bar( "$count. $target_html (rating: @{[$target->rating]})",
                             $deaths, [ $bar_length, $levels ] );
}

=item most_popular_targets_footer

=cut

sub most_popular_targets_footer {
  return "</table>\n";
}

=item most_popular_weapons_header NAME

The header for a players most popular weapons

=cut

sub most_popular_weapons_header {
  my ($name) = @_;

  $name = strip_colors($name);

  return qq(
        <table class="eghtpctpad">
        <tr>
          <th class="header" colspan="3">
            Favorite Weapons for $name
          </th>
        </tr>
        <tr>
          <th class="header">Weapon</th>
          <th class="header">Kills</th>
          <th class="header">Ratio of total Kills</th>
        </tr>
    );
}

=item most_popular_weapons_for_player COUNT WEAPON KILLS TOTAL_KILLS DEPTH

=cut

sub most_popular_weapons_for_player {
  my ( $count, $weapon, $kills, $total_kills, $depth ) = @_;

  my $bar_length = ( 100 * $kills ) / $total_kills;

  return stat_line_with_bar( "$count. $weapon", $kills,
                             [ $bar_length, $depth ] );
}

=item most_popular_weapons_footer

returns the html for the most popular weapons table

=cut

sub most_popular_weapons_footer {
  return "</table>\n";
}

################################################################################
# HELPER METHODS
################################################################################

=item convert_seconds_to_time_format SECONDS

Short: Converts seconds to hours, minutes and seconds
Input: $seconds -total seconds for player
Output: ex string "312h 46m 54s"

=cut

sub convert_seconds_to_time_format {
  my ($seconds) = @_;
  return 0 unless $seconds;
  my $hours = int( $seconds / 3600 );    #convert to hours
  $seconds %= 3600;                      #remove the hours we figured out
  my $minutes = int( $seconds / 60 );
  $seconds %= 60;
  return sprintf "%3dh %02dm %02ds", $hours, $minutes, $seconds;
}

=item alt_colors NUM

Short: alternates colors in the tables.  Sets a package level var and
alternates between returning cell1 and cell2.

=cut

sub alt_colors {
  my $num = int( $cell_num / 2 + 1 );
  if ( $num > 2 ) {
    $num      = 1;
    $cell_num = .99;
  }

  $cell_num += .99;
  return 'cell' . $num;
}

=item strip_colors PLAYER_NAME %COLOR_MAP

Short: Convert the Q3 color control codes to HTML format

=cut

sub strip_colors {
  my $player_name = shift;
  my $i           = 0;

  # convert symbols to their html counterparts
  $player_name =~ s/\^[\<\>]//g;
  $player_name =~ s/\</\&lt\;/g;
  $player_name =~ s/\>/\&gt\;/g;

  return unless $player_name;

  # two hats means a hat!!!
  $player_name =~ s/\^\^/\&#94/g;

  $player_name =~ s/\^(\S)([^\^]+)/<font color=\"$COLOR_MAP{$1}\">$2<\/font>/g;

  # Lastly, if we didn't catch it, strip it out
  $player_name =~ s/\^\S//g;

  return $player_name;
}

sub html_bar {
  my ( $bar_length, $levels ) = @_;

  die "Err: no image_dir in OutputHTML.pm" unless $image_dir;

  my $depth = $levels
    ? join '/', ( '..' x $levels )
    : '.';

  my $left_img  = "$depth/$image_dir/left.gif";
  my $main_img  = "$depth/$image_dir/main.gif";
  my $right_img = "$depth/$image_dir/right.gif";
  my $percent   = sprintf( "%.2f%%", $bar_length );
  $bar_length = int $bar_length;

  return qq(
  <img src="$left_img" height="10" alt="left"><img src="$main_img" height="10" width="$bar_length" alt="main"><img src="$right_img" height="10" alt="right">&nbsp;$percent
    );
}

# will return a link to the players name if they're not filtered,
# otherwise it just returns the player's name convereted to html
sub htmlize_and_link_player {
  my ( $player, $output_dir ) = @_;

  $output_dir =
    ( $output_dir && $output_dir !~ m|/$| )
    ? $output_dir . '/'
    : '';

  my $player_filename   = $output_dir . filename_for_player($player);
  my $color_player_name = strip_colors( $player->name );

  return $player->is_filtered
    ? $color_player_name
    : qq|<a href="$player_filename" class="player" >$color_player_name</a>|;
}

sub stat_line_with_bar {
  my (@rows) = @_;

  my $bar_info = pop @rows;

  my $html = qq/<tr class="@{[alt_colors()]}">\n/;
  $html .= html_format_rows(@rows);
  $html .= '<td>' . html_bar(@$bar_info) . '</td>';
  $html .= "\n</tr>\n";
  return $html;
}

sub html_format_rows {
  my (@rows) = @_;
  return join "\n", ( map { "<td>$_</td>" } @rows );
}

sub render_dataset {
  my ( $caption, $headers, @rows ) = @_;
  my $out = '';

  $out .= qq|<table class="dataset">|;
  $out .= qq|<caption>$caption</caption>|;
  $out .= "<tr>" . join( "\n", map { "<th>$_</th>" } @$headers ) . "</tr>";
  my $count = 0;
  foreach my $row (@rows) {
    my $cell_class =
      ( $count % 4 < 2 )
      ? 'cell1'
      : 'cell2';
    $out .=
      qq|<tr class="$cell_class">|
      . join( "\n", map { "<td>$_</td>" } @$row ) . "</tr>";
    $count++;
  }
  $out .= qq|</table>|;
  return $out;
}

1;
