package SOF2Logger::Parser::COD;

use strict;
use warnings;

use base 'SOF2Logger::Parser::SOF2';

use constant Utils => 'SOF2Logger::Utils';

# NOTES: it looks like we can determine the version based on the
# shortversion key in the init line.

# This module needs to be updated with both a 1.2/1 parser and the
# next gen 1.3/4 log.

# Unforutunately, I see no way to determine that we are running the
# ROC mod.

sub record_kill
{
    my ($self, $time, $info) = @_;

    # this is for parsing COD 1.3 and 1.4, but 1.2 and 1.1 are different
    my ($vic_uid,
        $vic_slot, 
        $vic_team, 
        $vic_name, 
        $aggr_uid,
        $aggr_slot,
        $aggr_team,
        $aggr_name,
        $weap_name,
        $damage,
        $dam_source,
        $loc) = split /;/, $info;
    my $vic_id = uc(Utils->clean_q3_color_string($vic_name));
    my $aggr_id = uc(Utils->clean_q3_color_string($aggr_name));

# TODO: Add tracking for team kills or world kills
    $self->kills->add_kill(
        $aggr_id ? $aggr_id : $self->world_id,
        $vic_id, 
        $weap_name eq 'none' ? $dam_source : $weap_name,
        $loc,
        $time
    );
}

sub change_info_client
{
# TODO: I haven't seen one of these yet...
}

sub _get_info_from_disconnect
{
    my ($self, $info) = @_;

    my ($slot, $name) = split /;/, $info;
    return $slot;
}

sub connect_client
{
    my ($self, $time, $info) = @_;

    my @info = split(/;/, $info, 3);

    # for certain versions of cod, we have a UID, for other versions we don't.
    # Therefore, if we have 3 elements, the first is the uid, if we only have 2,
    # the first element is the name
    my ($id, $slot, $name);
    if (@info == 2) {
        ($slot, $name) = @info;
        $id = uc(Utils->clean_q3_color_string($name));
    } else {
        ($id, $slot, $name) = @info;
    }

    warn "$id, $slot, $name\n\n";

    if (!$self->players($id)) {
        $self->players($id, SOF2Logger::Player->new($name, $id));
    }

    $self->new_player_at_slot($slot, $id, 0, $time);
}

sub add_flag_capture {}

sub _parse_line
{
    # 35:01 J;111222;0;{LLC}Bunga^7
    my ($self, $line) = @_;

    my ($time, $command, $info);

    if ($line =~ / *(\d+):(\d)(\d?)\s?([^;:]+)(;|:)(.*)/) {
        $time = ($1 * 60) + ($2 * 10) + ($3?$3:0);
        $command = $4;
        $info = $6;
    }

    return $time, $command, $info;
}

sub attach_methods_to_hash_keys
{
    my ($self) = @_;

    $self->events({
        InitGame => 'init_game',
        J => 'connect_client',
        K => 'record_kill',
        Q => 'disconnect_client',
        ExitLevel => 'shutdown_game',
        RestartGame => 'shutdown_game',});
}

=head 

Line Structure
    (\d+)  #minutes
    :
    (\d\d) #seconds (sometimes missing second \d)
    (\w)   #event (K, J, W, L, D, RestartGame, InitGame)
    ;      #token seperator

for join or quit lines:
    after event
    (\d+)  #player number
    ;
    (.*)  # player name

for kill or damage lines:
    after event
    (\d+)  #victim number
    ;
    (axis|allies) #victim team
    ;
    ([^;]+)       #victim name
    ;
    (\d+)         #aggressor player number
    ;
    (axis|allies) #aggressor team
    ;
    ([^;]+)       #aggressor name
    ;
    ([^;]+)       #weapon name
    ;
    (\d+)         #amount of damage done by attack
    ;
    (MOD_[A-Z]+)  #death reason (PISTOL_BULLET, MOD_HEAD_SHOT, GRENADE_SPLASH)
    ;
    (.+)          #location (can be 'none'.  or, right_leg_lower, head, right_foot, etc)

for win/lose lines
    after event:
    (axis|allies)  #team
    (;[^;]+)+      #all player names on that team, seperated by ';'

connect lines (I'm assuming J means 'join')

3339:18J;15;^4N^7o^4o^7B^4i^7E
3339:18J;14;OneShot=OneKill
3339:18J;13;Bender
3339:18J;12;IronRabbit
3339:18J;11;pvt. Idaho  C
3339:18J;10;{llc}$|ICKGREEN
3339:18J;9;}I{ Battlemage
3339:18J;8;bLanK
3339:18J;7;^3{LLC}The^7 ^3Wasp
3339:18J;6;{LLC}Andyman
3339:18J;5;^2{^1LLC^2}^1HA^3MM^1ERS
3339:18J;4;^6ti.Spectre
3339:18J;3;^7^4t^7i. Obsidian
3339:18J;2;^1Mickey^4SPILLANE
3339:18J;1;^3{^4LLC^3}^4klaypigeon
3339:18J;0;{LLC}GETSUM

Quit line:
3354:19Q;14;^2AC^3Delta

Weapon purchases?  Weapon use?
3339:23Weapon: 3 panzerfaust_mp

Talking lines
3339:35say: {LLC}klaypigeon: ya
3339:38say: {LLC}klaypigeon: :(

Damage but not a kill?
3339:51D;6;axis;{LLC}Andyman;12;allies;IronRabbit;bar_mp;56;MOD_RIFLE_BULLET;torso_lower
3339:52D;6;axis;{LLC}Andyman;12;allies;IronRabbit;bar_mp;42;MOD_RIFLE_BULLET;right_leg_upper

Kills?
3340:01K;1;allies;^3{^4LLC^3}^4klaypigeon;0;axis;{LLC}GETSUM;fg42_mp;62;MOD_RIFLE_BULLET;torso_upper
3340:04K;11;allies;pvt. Idaho  C;13;axis;Bender;mp44_mp;34;MOD_PISTOL_BULLET;left_arm_lower
3340:04K;14;allies;OneShot=OneKill;3;axis;^7^4t^7i. Obsidian;kar98k_sniper_mp;180;MOD_HEAD_SHOT;head

Won round?
3341:11W;axis;{LLC}GETSUM;^1Mickey^4SPILLANE;^7^4t^7i. Obsidian;{LLC}Andyman;bLanK;{llc}$|ICKGREEN;Bender;^4N^7o^4o^7B^4i^7E

Lost Round?
3341:11L;allies;^3{^4LLC^3}^4klaypigeon;^6ti.Spectre;^2{^1LLC^2}^1HA^3MM^1ERS;}I{ Battlemage;pvt. Idaho  C;IronRabbit;OneShot=OneKill

Bomb plant
3344:46A;9;allies;}I{ Battlemage;bomb_plant

Bomb diffues
3345:20A;2;axis;^1Mickey^4SPILLANE;bomb_defuse

Exit level?
3357:58ExitLevel: executed

Restart Game?
3341:16RestartGame:

=cut
1;
