package SOF2Logger::Parser::JKII;

use strict;
use warnings;

use base qw/SOF2Logger::Parser::SOF2/;

# Jedi Academy : JK3?

sub attach_methods_to_hash_keys
{
    my ($self) = @_;

    $self->events(
        {
            'InitGame' => 'init_game',
            'ClientConnect' => 'connect_client',
            'Kill' => 'record_kill',
            'ShutdownGame' => 'shutdown_game',
            'ClientDisconnect' => 'disconnect_client',
            'setteam' => 'change_info_client',
        },
    );
}

# 1255:32ClientConnect: 1
sub connect_client
{
    return 0;
}

sub record_kill
{
    my ($self, $time, $line_data) = @_;

    my ( $short_info, $long_info ) = split /:/, $line_data, 2;
    my ( $killer_slot, $victim_slot, $weapon_num ) = split ' ', $short_info, 3;

    $long_info =~ / (.*?) killed (.*?) by (MOD_\w+)/;
    my $killer_name = $1;
    my $killer_id = uc(Utils->clean_q3_color_string($killer_name));
    my $victim_name = $2;
    my $victim_id = uc(Utils->clean_q3_color_string($victim_name));
    my $weapon_name = $3;

    $self->new_player_at_slot($killer_slot, $killer_id, 'FREE', $time, $killer_name);
    $self->new_player_at_slot($victim_slot, $victim_id, 'FREE', $time, $victim_name);

    # bail if we don't have all 3 pieces of information
    return if !$killer_id || !$victim_id || !$self->weapon_map($weapon_name);

    # potential suicide
    if ( $weapon_num == $SUICIDE || $killer_slot == $victim_slot ) {
            $self->kills->add_suicide( $victim_id, $time_in_seconds );
    } else {
        $self->kills->add_kill(
            $killer_id,
            $victim_id,
            $self->weapon_map($weapon_name),
            'none',
            $time_in_seconds
        );
    }

    $self->last_known_good_time($time_in_seconds);
}

sub _add_new_player_from_info
{
    my ($self, $id, $name) = @_;
    $self->players(
        {
            $id => SOF2Logger::Player->new($name, $id),
        },
    );
}

# 1255:47setteam:  1 SPECTATOR FREE
sub change_info_client
{
    my ($self, $time, $line_data) = @_;

    my ($slot, $from_team, $to_team) = $line_data =~ /(\d+)\s+(\w+)\s+(\w+)/;
    my $game_slot = $self->game_slots_index($slot);

    # if moving to a fighting team
    if ( $to_team ne 'SPECTATOR' ) {
        $self->game_slots_index($slot)->{START_TIME}   = $time;
        $self->game_slots_index($slot)->{IS_SPECTATOR} = 0;
    } elsif ($from_team ne 'SPECTATOR') {
        # moving to spectator team, not from spectator team
        $self->stop_time( $slot, $time );
        $self->game_slots_index($slot)->{IS_SPECTATOR} = 1;
    }
}

sub init_game
{
}

sub shutdown_game
{
}

sub change_info_client
{
}

sub disconnect_client
{
}

sub connect_client
{
}

sub init_color_map
{
    my ($self) = @_;
    $self->color_map(
        '0' => "#000000",    # BLACK
        '1' => "#FF0000",    # RED
        '2' => "#80FF00",    # GREEN
        '3' => "#FFFF00",    # YELLOW
        '4' => "#0000FF",    # BLUE
        '5' => "#00FFFF",    # CYAN
        '6' => "#FF00FF",    # MAGENTA
        '7' => "#FFFFFF",    # WHITE
        '8' => "#AD19D5",    # PURPLE
        '9' => "#A7F4F1",    # CYAN

        'a' => "#48E2DC",    # Cyan
        'b' => "#FAE3A9",    # Orange
        'c' => "#E5A7E5",    # Pink
        'd' => "#DEF7FE",    # Blue
        'e' => "#D71BBC",    # Pink
        'f' => "#CFFAFB",    # Blue
        'g' => "#D1F783",    # Green
        'h' => "#A91859",    # Red
        'i' => "#789BCB",    # Blue
        'j' => "#9BEE44",    # Green
        'k' => "#FEFCDA",    # Tan
        'l' => "#EEFBFD",    # Blue
        'm' => "#DB79DA",    # Pink
        'n' => "#ACC997",    # Green
        'o' => "#E9C9CB",    # Pink
        'p' => "#84D6C0",    # Cyan
        'q' => "#FAEEF1",    # Tan
        'r' => "#E5A7E5",    # Pink
        's' => "#6115A3",    # Blue
        't' => "#B31164",    # Red
        'u' => "#846FD0",    # Blue
        'v' => "#5BE593",    # Green
        'w' => "#000000",    # Black
        'x' => "#0C2C4C",    # Blue
        'y' => "#48E2DC",    # Blue
        'z' => "#EFD497",    # Orange
        'A' => "#7918CE",    # Purple
        'B' => "#9CED8B",    # Green
        'C' => "#E3AA87",    # Orange
        'D' => "#889CB8",    # Blue
        'E' => "#CEEF99",    # Green
        'F' => "#1F5C9A",    # Blue
        'G' => "#A8EB95",    # Green
        'H' => "#D5CCCC",    # Pink
        'I' => "#C0F0F0",    # Blue
        'J' => "#F5FCEF",    # Gray
        'K' => "#F6FEDC",    # Green
        'L' => "#F1DAF7",    # Purple
        'M' => "#44E0BA",    # Cyan
        'N' => "#EBDA6F",    # Orange
        'O' => "#FFFFFF",    # White
        'P' => "#35D936",    # Green
        'Q' => "#F79DF1",    # Pink
        'R' => "#7686D7",    # Blue
        'S' => "#9CED8B",    # Green
        'T' => "#DDAB9F",    # Orange
        'U' => "#CDA3E5",    # Purple
        'V' => "#DBFAF8",    # Gray
        'W' => "#9F25B8",    # Purple
        'X' => "#6FF2EE",    # Cyan
        'Y' => "#D4F353",    # Green
        'Z' => "#F8EEFD",    # Purple

        '!'  => "#000000",   # BLACK
        '@'  => "#F7D43B",   # ORANGE
        '#'  => "#000000",   # BLACK
        '$'  => "#C71819",   # RED
        '&'  => "#A39DC5",   # BLUE
        '*'  => "#F7FCFB",   # WHITE
        '('  => "#CD4624",   # PINK
        ')'  => "#40E3E7",   # CYAN
        '_'  => "#A82526",   # RED
        '-'  => "#8A8789",   # GRAY
        '+'  => "#FFFFFF",   # WHITE
        '='  => "#BED4C9",   # CYAN
        '{'  => "#CD4624",   # RED
        '['  => "#A5EEE1",   # CYAN
        '}'  => "#6E939B",   # BLUE
        ']'  => "#89EEE4",   # CYAN
        '|'  => "#D6F7B5",   # GREEN
        '\\' => "#D6ABCE",   # PINK
        ':'  => "#FFFFFF",   # WHITE
        '\'' => "#8A8789",   # GRAY
        '<'  => "#2CE02F",   # GREEN
        ','  => "#DDE8D6",   # GRAY
        '>'  => "#81F2F1",   # CYAN
        '.'  => "#A39DC5",   # BLUE
        '?'  => "#D6F7B5",   # GREEN
        '/'  => "#411959"    # PURPLE
    );
}

sub init_weapon_map
{
    my ($self) = @_;
    $self->weapon_map(
        "MOD_SABER" 			=> 	"Lightsaber",
        "MOD_BLASTER"			=>	"Blaster",
        "MOD_BOWCASTER"			=>	"Bowcaster",
        "MOD_BRYAR_PISTOL" 		=>	"Bryar Pistol",
        "MOD_BRYAR_PISTOL_ALT" 		=> 	"Bryar Pistol Alternate Fire",
        "MOD_CRUSH"			=>	"Crush",
        "MOD_DEMP2"			=>	"MOD_DEMP2",
        "MOD_DET_PACK_SPLASH" 		=>	"Detonation Pack Splash Damage",
        "MOD_DISRUPTOR"			=>	"Disruptor",
        "MOD_DISRUPTOR_SNIPER" 		=> 	"Disruptor Sniper Fire",
        "MOD_FALLING"			=>	"Falling",
        "MOD_FLECHETTE"			=>	"Flechette",
        "MOD_FLECHETTE_ALT_SPLASH"	=> 	"Flechette Alternate Splash Damage",
        "MOD_FORCE_DARK"		=>	"Dark Force",
        "MOD_MELEE"			=>	"Melee",
        "MOD_REPEATER"			=>	"Repeater",
        "MOD_REPEATER_ALT"		=>	"Repeater Alternate Fire",
        "MOD_REPEATER_ALT_SPLASH"	=>	"Repeater Alternate Splash Damage",
        "MOD_ROCKET"			=>	"Rocket",
        "MOD_ROCKET_HOMING"		=>	"Rocket Homing Mode",
        "MOD_ROCKET_HOMING_SPLASH"	=>	"Rocked Homing Splash Damage",
        "MOD_ROCKET_SPLASH"		=>	"Rocket Splash Damage",
        "MOD_SENTRY"			=>	"Sentry Gun",
        "MOD_STUN_BATON"		=>	"Stun Baton",
        "MOD_SUICIDE"			=>	"Suicide",
        "MOD_TELEFRAG"			=>	"Telefrag",
        "MOD_THERMAL"			=>	"Thermal Detonator",
        "MOD_THERMAL_SPLASH"	=>	"Thermal Detonator Splash Damage",	
        "MOD_TRIGGER_HURT"		=>	"Trigger Hurt",		
        "MOD_TRIP_MINE_SPLASH"	=>	"Trip Mine Splash Damage",
        "MOD_UNKNOWN"			=>	"Unknown",
    );
}

1;
