package SOF2Logger::Parser::SOF2::ROC;

use strict;

use base qw/SOF2Logger::Parser::SOF2/;

use SOF2Logger::Parser::SlotList;

sub get_events {
    my $self = shift;
    my $events =  $self->SUPER::get_events();
    $events->{'Flag Capture'} = 'add_flag_capture';
    return $events;
}

sub is_headshot_kill
{
    my ($self, $long_info) = @_;

    return $long_info =~ /at hit location:1024 onsameteam:.$/;
}

sub add_flag_capture
{
    my ($self, $time_in_seconds, $data) = @_;
    my ($short_info, $long_info) = split /:/, $data, 2;
    my ($flag_type, $player_slot) = $short_info =~ /(\d+) (\d+)/;
    SOF2Logger::Parser::SlotList->get_slot($player_slot)->player->add_cap;
}

1;

=pod

FROM FILE

my %WEAPON_MAP = (
    "MOD_KNIFE [HEADSHOT]"                   => "Knife - [HEADSHOT]",
    "MOD_KNIFE_ALT [HEADSHOT]"               => "Knife Throw - [HEADSHOT]",
    "MOD_US_SOCOM_PISTOL_ALT [HEADSHOT]"     => "SOCOM: Head Smash - [HEADSHOT]",
    "MOD_US_SOCOM_PISTOL [HEADSHOT]"         => "SOCOM - [HEADSHOT]",
    "MOD_M1911A1_PISTOL_ALT [HEADSHOT]"      => "M1911-A1: Head Smash - [HEADSHOT]",
    "MOD_M1911A1_PISTOL [HEADSHOT]"          => "M1911-A1 - [HEADSHOT]",
    "MOD_SMOHG92_GRENADE [HEADSHOT]"         => "Frag Nade - [HEADSHOT]",
    "MOD_SMOHG92_GRENADE_ALT [HEADSHOT]"     => "Frag Nade Rolled - [HEADSHOT]",
    "MOD_M84_GRENADE [HEADSHOT]"             => "Flash Nade - [HEADSHOT]",
    "MOD_M84_GRENADE_ALT [HEADSHOT]"         => "Flash Nade Rolled - [HEADSHOT]",
    "MOD_ANM14_GRENADE [HEADSHOT]"           => "Fire Nade - [HEADSHOT]",
    "MOD_ANM14_GRENADE_ALT [HEADSHOT]"       => "Fire Nade Rolled - [HEADSHOT]",
    "MOD_M15_GRENADE [HEADSHOT]"             => "Smoke Nade - [HEADSHOT]",
    "MOD_M15_GRENADE_ALT [HEADSHOT]"         => "Smoke Nade Rolled - [HEADSHOT]",
    "MOD_M590_SHOTGUN [HEADSHOT]"            => "Shottie - [HEADSHOT]",
    "MOD_M590_SHOTGUN_ALT [HEADSHOT]"        => "Shottie Butt - [HEADSHOT]",
    "MOD_USAS_12_SHOTGUN [HEADSHOT]"         => "Auto Shottie - [HEADSHOT]",
    "MOD_SIG551 [HEADSHOT]"                  => "SIG551 - [HEADSHOT]",
    "MOD_SILVER_TALON [HEADSHOT]"            => "Silver Talon - [HEADSHOT]",
    "MOD_SILVER_TALON_ALT [HEADSHOT]"        => "Silver Talon: Head Smash - [HEADSHOT]",
    "MOD_AK74_ASSAULT_RIFLE [HEADSHOT]"      => "AK74 - [HEADSHOT]",
    "MOD_AK74_ASSAULT_RIFLE_ALT [HEADSHOT]"  => "AK74 Stab - [HEADSHOT]",
    "MOD_M4_ASSAULT_RIFLE [HEADSHOT]"        => "M4 - [HEADSHOT]",
    "MOD_M4_ASSAULT_RIFLE_ALT [HEADSHOT]"    => "M4 Nade - [HEADSHOT]",
    "MOD_TRIGGER_HURT_NOSUICIDE [HEADSHOT]"  => "Killed By Dem Bomb - [HEADSHOT]",
    "MOD_WATER [HEADSHOT]"                   => "Drowned - [HEADSHOT]",
    "MOD_TRIGGER_HURT [HEADSHOT]"            => "Cratered - [HEADSHOT]",
    "MOD_SUICIDE [HEADSHOT]"                 => "Suicide - [HEADSHOT]",
    "MOD_FALLING [HEADSHOT]"                 => "Ground Fall - [HEADSHOT]",
    "MOD_CRUSH [HEADSHOT]"                   => "Crushed - [HEADSHOT]",
    "MOD_TELEFRAG [HEADSHOT]"                => "TeleFraged - [HEADSHOT]",
    "MOD_UNKNOWN [HEADSHOT]"                 => "Unknown - [HEADSHOT]",
    "MOD_TEAMCHANGE [HEADSHOT]"              => "Team Change - [HEADSHOT]",
    "MOD_RPG7_LAUNCHER [HEADSHOT]"           => "RPG Launcher - [HEADSHOT]",
    "MOD_MM1_GRENADE_LAUNCHER [HEADSHOT]"    => "MM1 Grenade Launcher - [HEADSHOT]",
    "MOD_M3A1_SUBMACHINEGUN [HEADSHOT]"      => "M3A1 SMG - [HEADSHOT]",
    "MOD_MSG90A1_SNIPER_RIFLE [HEADSHOT]"    => "Sniper Rifle - [HEADSHOT]",
    "MOD_MICRO_UZI_SUBMACHINEGUN [HEADSHOT]" => "Micro Uzi - [HEADSHOT]",
    "MOD_M60_MACHINEGUN [HEADSHOT]"          => "M60 MG - [HEADSHOT]",
    "MOD_MP5 [HEADSHOT]"                     => "MP5 - [HEADSHOT]",
    "MOD_STRIKE [HEADSHOT]"                  => "ROC mod Admin Strike - [HEADSHOT]",
    "MOD_FLAGCAP"                            => "Flag Captures",
    "MOD_KNIFE"                              => "Knife",
    "MOD_KNIFE_ALT"                          => "Knife Throw",
    "MOD_US_SOCOM_PISTOL_ALT"                => "SOCOM: Head Smash",
    "MOD_US_SOCOM_PISTOL"                    => "SOCOM",
    "MOD_M1911A1_PISTOL_ALT"                 => "M1911-A1: Head Smash",
    "MOD_M1911A1_PISTOL"                     => "M1911-A1",
    "MOD_SMOHG92_GRENADE"                    => "Frag Nade",
    "MOD_SMOHG92_GRENADE_ALT"                => "Frag Nade Rolled",
    "MOD_M84_GRENADE"                        => "Flash Nade",
    "MOD_M84_GRENADE_ALT"                    => "Flash Nade Rolled",
    "MOD_ANM14_GRENADE"                      => "Fire Nade",
    "MOD_ANM14_GRENADE_ALT"                  => "Fire Nade Rolled",
    "MOD_M15_GRENADE"                        => "Smoke Nade",
    "MOD_M15_GRENADE_ALT"                    => "Smoke Nade Rolled",
    "MOD_M590_SHOTGUN"                       => "Shottie",
    "MOD_M590_SHOTGUN_ALT"                   => "Shottie Butt",
    "MOD_USAS_12_SHOTGUN"                    => "Auto Shottie",
    "MOD_SIG551"                             => "SIG551",
    "MOD_SILVER_TALON"                       => "Silver Talon",
    "MOD_SILVER_TALON_ALT"                   => "Silver Talon: Head Smash",
    "MOD_AK74_ASSAULT_RIFLE"                 => "AK74",
    "MOD_AK74_ASSAULT_RIFLE_ALT"             => "AK74 Stab",
    "MOD_M4_ASSAULT_RIFLE"                   => "M4",
    "MOD_M4_ASSAULT_RIFLE_ALT"               => "M4 Nade",
    "MOD_TRIGGER_HURT_NOSUICIDE"             => "Killed By Dem Bomb",
    "MOD_WATER"                              => "Drowned",
    "MOD_TRIGGER_HURT"                       => "Cratered",
    "MOD_SUICIDE"                            => "Suicide",
    "MOD_FALLING"                            => "Ground Fall",
    "MOD_CRUSH"                              => "Crushed",
    "MOD_TELEFRAG"                           => "TeleFraged",
    "MOD_UNKNOWN"                            => "Unknown",
    "MOD_TEAMCHANGE"                         => "Team Change",
    "MOD_RPG7_LAUNCHER"                      => "RPG Launcher",
    "MOD_MM1_GRENADE_LAUNCHER"               => "MM1 Grenade Launcher",
    "MOD_M3A1_SUBMACHINEGUN"                 => "M3A1 SMG",
    "MOD_MSG90A1_SNIPER_RIFLE"               => "Sniper Rifle",
    "MOD_MICRO_UZI_SUBMACHINEGUN"            => "Micro Uzi",
    "MOD_M60_MACHINEGUN"                     => "M60 MG",
    "MOD_MP5"                                => "MP5",
    "MOD_STRIKE"                             => "ROC mod Admin Strike");
=cut
