#!/usr/bin/perl -w

use strict;

use Test::More tests => 7;

use SOF2Logger::Parser::SOF2::ROC;
use SOF2Logger::Player;

ok(SOF2Logger::Parser::SOF2::ROC->is_headshot_kill('at hit location:1024 onsameteam:0'),
    'is headshot line');

ok(!SOF2Logger::Parser::SOF2::ROC->is_headshot_kill(
    '*GWJ* Foo Bar [HEADSHOT] GWJ* ThePhoeniX CL by MOD_M590_SHOTGUN'),
    'is not headshot line');

ok(SOF2Logger::Parser::SOF2::ROC->is_headshot_kill(
    '*GWJ* ThE HiLL|DK killed *GWJ* ThePhoeniX CL by MOD_M590_SHOTGUN at hit location:1024 onsameteam:0'),
    'is headshot line');

my $parser = create_test_players_in_slots();

$parser->add_flag_capture(1033, '1 1: hottie');
is(SOF2Logger::Player->find('00000001')->flag_caps, 1, 'flag caps for hottie');
my ($time, $event, $info) = $parser->parse_line('80:56 Flag Capture: 1 0: Whoever');
    is($time, 80 * 60 + 56, 'time');
is($event, 'Flag Capture', 'event');
is($info, '1 0: Whoever', 'info');


sub create_test_players_in_slots
{
    my $p1 = '1 n\hottie\t\3\identity\marinesoldier1\sir\0\sib\0\grade\0';
    my $p2 = '2 n\billy\t\2\identity\marinesoldier1\sir\0\sib\0\grade\0';
    my $p3 = '3 n\bobby\t\1\identity\marinesoldier1\sir\0\sib\0\grade\0';
    my $test_parser = SOF2Logger::Parser::SOF2::ROC->new('logfile.log', 0);

    $test_parser->{game_version} = '1_02';

    $test_parser->connect_client(933, '1 - 10.0.0.1:1025 [00000000000000000000000000000001]');
    $test_parser->connect_client(953, '2 - 10.0.0.1:1025 [00000000000000000000000000000002]');
    $test_parser->connect_client(953, '3 - 10.0.0.1:1025 [00000000000000000000000000000003]');
    $test_parser->change_info_client(933, $p1);
    $test_parser->change_info_client(953, $p2);
    $test_parser->change_info_client(953, $p3);

    return $test_parser;
}

