package SOF2Logger::Parser::SlotList;

# SlotList : A smart data structure
#
# SlotList tries to contain the logic for controlling the game slots,
# which should be generally useful for all games.

use strict;
use warnings;

use SOF2Logger::Parser::Slot;

my @slot_list;

# everything in here is a class method

# Changes the slots information,
sub put_player_in_slot {
  my ($class, $num, $player, $team, $time, $is_spectator) = @_;
  my $slot = $slot_list[$num];

  $class->disconnect($num, $time) if $slot;

  $slot = $slot_list[$num] = SOF2Logger::Parser::Slot->new();
  $slot->player($player);
  $slot->team($team);
  $slot->start_time($time);
  $slot->is_spectator($is_spectator);
  $slot->num($num);

  return $slot;
}

# if we have created a slot, we should always have a player, and
# when we disconnect that slot, we delete the slot.

# Since our logfiles can have multiple sequential disconnects, we
# should not die when we try to disconnect a slot that doesn't
# exist, we just skip over it and keep going
sub disconnect {
    my ($class, $num, $time) = @_;
    my $slot = $class->get_slot($num);
    return unless $slot;

    $slot->player->add_time($class->stop_time($slot, $time));
    $slot_list[$num] = undef;
}

sub stop_time {
    my ($class, $slot, $stop_time) = @_;

    # its possible that we're trying to stop a slot that shouldn't
    # really be stopped, so we just return 0
    return 0 if ( !$slot
		  || !$slot->player()
		  || $slot->is_spectator
		  || $slot->start_time() == -1
		  || $slot->start_time > $stop_time );

    return $stop_time - $slot->start_time();
}

sub get_slot {
  my ($class, $num) = @_;
  return $slot_list[$num];
}

sub get_player {
  my ($class, $num) = @_;
  return $slot_list[$num] && $slot_list[$num]->player;
}

sub all_slots {
  return grep { defined $_ } @slot_list;
}

1;
