#!/usr/bin/perl -w

use strict;

# use Test::More tests => 85;
###### skipping all tests until we can figure out what the logs are supposed to look like ###
use Test::More skip_all => 'pending new log files';

use SOF2Logger::Parser::COD;
use File::Basename qw/dirname/;

test_parse_log();
test_parse_line();
test_connect_creates_player_in_players();
test_records_world_kills_properly();


my $parser = SOF2Logger::Parser::COD->new('cod_log.log', 0);
push @{$parser->game_slots}, { foo => 'bar'};
push @{$parser->game_slots}, { bar => 'baz'};
push @{$parser->game_slots}, { baz => 'bee'};

is(@{$parser->game_slots}, 3, 'len of game slots');

sub test_parse_log
{
    my $parser = SOF2Logger::Parser::COD->new(dirname(__FILE__) . '/cod_test.log', 0);
    $parser->parse_log();

    my $players = {
        '{LLC}Andyman' => {
            'flag_caps' => 0,
            'total_time' => 118,
            'id' => 'LLCANDYMAN',
        },
        '^3{^4LLC^3}^4klaypigeon' => {
            'flag_caps' => 0,
            'total_time' => 118,
            'id' => 'LLCKLAYPIGEON',
        },
        '^4N^7o^4o^7B^4i^7E' => {
            'flag_caps' => 0,
            'total_time' => 118,
            'id' => 'NOOBIE',
        },
        '^2{^1LLC^2}^1HA^3MM^1ERS' => bless( {
            'flag_caps' => 0,
            'total_time' => 118,
            'id' => 'LLCHAMMERS',
        }, 'SOF2Logger::Player' ),
        '^7^4t^7i. Obsidian' => bless( {
            'flag_caps' => 0,
            'total_time' => 118,
            id => 'TIOBSIDIAN',
        }, 'SOF2Logger::Player' ),
        '^6ti.Spectre' => bless( {
            'flag_caps' => 0,
            'total_time' => 118,
            id => 'TISPECTRE',
        }, 'SOF2Logger::Player' ),
        '^3{LLC}The^7 ^3Wasp' => bless( {
            'flag_caps' => 0,
            'total_time' => 118,
            id => 'LLCTHEWASP',
        }, 'SOF2Logger::Player' ),
        'bLanK' => bless( {
            'flag_caps' => 0,
            'total_time' => 118,
            id => 'BLANK',
        }, 'SOF2Logger::Player' ),
        'Bender' => bless( {
            'flag_caps' => 0,
            'total_time' => 118,
            id => 'BENDER',
        }, 'SOF2Logger::Player' ),
        'OneShot=OneKill' => bless( {
            'flag_caps' => 0,
            'total_time' => 118,
            id => 'ONESHOTONEKILL',
        }, 'SOF2Logger::Player' ),
        '{llc}$|ICKGREEN' => bless( {
            'flag_caps' => 0,
            'total_time' => 118,
            id => 'LLCICKGREEN',
        }, 'SOF2Logger::Player' ),
        '}I{ Battlemage' => bless( {
            'flag_caps' => 0,
            'total_time' => 118,
            id => 'IBATTLEMAGE',
        }, 'SOF2Logger::Player' ),
        'IronRabbit' =>bless( {
            'flag_caps' => 0,
            'total_time' => 118,
            id => 'IRONRABBIT',
        }, 'SOF2Logger::Player' ),
        '^1Mickey^4SPILLANE' => bless( {
            'flag_caps' => 0,
            'total_time' => 118,
            id => 'MICKEYSPILLANE',
        }, 'SOF2Logger::Player' ),
        'pvt. Idaho  C' => bless( {
            'flag_caps' => 0,
            'total_time' => 118,
            id => 'PVTIDAHOC',
        }, 'SOF2Logger::Player' ),
        '{LLC}GETSUM' => bless( {
            'flag_caps' => 0,
            'total_time' => 118,
            id => 'LLCGETSUM',
        }, 'SOF2Logger::Player' ),
        '<world>' => bless( {
            'flag_caps' => 0,
            'total_time' => 0,
            id => 'WORLD',
        }, 'SOF2Logger::Player' )
    };

    use Data::Dumper;
    die Dumper $parser->players;

    foreach my $player (keys %$players) {
        # name
        is( $parser->players($players->{$player}{id})->name,
            $player,
            "player $player name");
        # flag caps
        is( $parser->players($players->{$player}{id})->flag_caps,
            $players->{$player}{flag_caps},
            "player $player flag_caps");
        # total time
        is( $parser->players($players->{$player}{id})->total_time,
            $players->{$player}{total_time},
            "player $player total time");
        # id
        is( $parser->players($players->{$player}{id})->id,
            $players->{$player}{id},
            "player $player id");
    }

my $kills_ref = {
    'BY_KILLER_BY_WEAPON' => {
        'TISPECTRE' => {
            'thompson_mp' => 1,
        },
        'BENDER' => {
            'mp44_mp' => 1,
        },
        'NOOBIE' => {
            'mp44_mp' => 1,
        },
        'IRONRABBIT' => {
            'bar_mp' => 1,
        },
        'MICKEYSPILLANE' => {
            'fg42_mp' => 1,
        },
        'TIOBSIDIAN' => {
            'm1carbine_mp' => 1,
            'kar98k_sniper_mp' => 3,
        },
        'LLCGETSUM' => {
            'fg42_mp' => 1,
        }
    },
    'IS_FILTERED_PLAYER' => {},
    'SUICIDES_BY_PLAYER' => {},
    'BY_KILLER' => {
        'TISPECTRE' => 1,
        'BENDER' => 1,
        'NOOBIE' => 1,
        'IRONRABBIT' => 1,
        'MICKEYSPILLANE' => 1,
        'TIOBSIDIAN' => 4,
        'LLCGETSUM' => 1,
    },
    'BY_KILLER_BY_LOCATION' => {
        'TISPECTRE' => {
            'head' => 1,
        },
        'BENDER' => {
            'left_arm_lower' => 1,
        },
        'NOOBIE' => {
            'head' => 1,
        },
        'IRONRABBIT' => {
            'torso_upper' => 1,
        },
        'MICKEYSPILLANE' => {
            'torso_lower' => 1,
        },
        'TIOBSIDIAN' => {
            'torso_upper' => 3,
            'head' => 1,
        },
        'LLCGETSUM' => {
            'torso_upper' => 1,
        }
    },
    'total_kills' => 10,
    'BY_KILLER_BY_VICTIM' => {
        'TISPECTRE' => {
            'BENDER' => 1,
        },
        'BENDER' => {
            'PVTIDAHOC' => 1,
        },
        'NOOBIE' => {
            'IBATTLEMAGE' => 1,
        },
        'IRONRABBIT' => {
            'LLCANDYMAN' => 1,
        },
        'MICKEYSPILLANE' => {
            'IRONRABBIT' => 1,
        },
        'TIOBSIDIAN' => {
            'LLCTHEWASP' => 1,
            'TISPECTRE' => 1,
            'ONESHOTONEKILL' => 1,
            'LLCHAMMERS' => 1,
        },
        'LLCGETSUM' => {
            'LLCKLAYPIGEON' => 1,
        }
    },
    'BY_WEAPON_BY_KILLER' => {
        'mp44_mp' => {
            'BENDER' => 1,
            'NOOBIE' => 1,
        },
        'bar_mp' => {
            'IRONRABBIT' => 1,
        },
        'm1carbine_mp' => {
            'TIOBSIDIAN' => 1,
        },
        'thompson_mp' => {
            'TISPECTRE' => 1,
        },
        'kar98k_sniper_mp' => {
            'TIOBSIDIAN' => 3,
        },
        'fg42_mp' => {
            'MICKEYSPILLANE' => 1,
            'LLCGETSUM' => 1,
        }
    },
    'BY_VICTIM_BY_KILLER' => {
        'LLCANDYMAN' => {
            'IRONRABBIT' => 1,
        },
        'LLCKLAYPIGEON' => {
            'LLCGETSUM' => 1,
        },
        'LLCHAMMERS' => {
            'TIOBSIDIAN' => 1,
        },
        'TISPECTRE' => {
            'TIOBSIDIAN' => 1,
        },
        'LLCTHEWASP' => {
            'TIOBSIDIAN' => 1,
        },
        'BENDER' => {
            'TISPECTRE' => 1,
        },
        'ONESHOTONEKILL' => {
            'TIOBSIDIAN' => 1,
        },
        'IBATTLEMAGE' => {
            'NOOBIE' => 1,
        },
        'IRONRABBIT' => {
            'MICKEYSPILLANE' => 1,
        },
        'PVTIDAHOC' => {
            'BENDER' => 1,
        }
    },
    'DEATHS_BY_PLAYER' => {
        'LLCANDYMAN' => 1,
        'LLCKLAYPIGEON' => 1,
        'LLCHAMMERS' => 1,
        'TISPECTRE' => 1,
        'LLCTHEWASP' => 1,
        'BENDER' => 1,
        'ONESHOTONEKILL' => 1,
        'IBATTLEMAGE' => 1,
        'IRONRABBIT' => 1,
        'PVTIDAHOC' => 1,
        },
    'total_deaths' => 10,
    'BY_WEAPON' => {
        'mp44_mp' => 2,
        'bar_mp' => 1,
        'm1carbine_mp' => 1,
        'thompson_mp' => 1,
        'kar98k_sniper_mp' => 3,
        'fg42_mp' => 2,
    }
};

#    is_deeply($kills_ref, scalar $parser->kills, 'kill is correct');
}

sub test_parse_line
{
    my $parser = SOF2Logger::Parser::COD->new('cod_log.log', 0);
    my ($time, $event, $info) = 
        $parser->_parse_line(' 35:01 J;0;{LLC}Bunga^7');
    is($time, 35 * 60 + 1, 'time in seconds');
    is($event, 'J', 'player join key');
    is($info, '0;{LLC}Bunga^7', 'event info');

    # need to add test for new 1.3 style join
    # time:sc Action ; <unique id?> ; slot ; name
    # 2293:43J;107175;0;=|Mars|=^CoNAn

    ($time, $event, $info) = $parser->_parse_line(' 36:45 RestartGame:');
    is($time, 36 * 60 + 45, 'time in seconds');
    is($event, 'RestartGame', 'restart game key');
    is($info, '', 'event info');

    ($time, $event, $info) = $parser->_parse_line('12345:4RestartGame:');
    is($time, 12345 * 60 + 40, 'squished time in seconds');
    is($event, 'RestartGame', 'restart game key');
    is($info, '', 'event info');

    ($time, $event, $info) = $parser->_parse_line(
        '3339:17InitGame: \.Admin\{LLC}klaypigeon\.Email\klaypigeon@hotmail');
    is($time, 3339 * 60 + 17, 'time in seconds');
    is($event, 'InitGame', 'init game event');
    is($info, ' \.Admin\{LLC}klaypigeon\.Email\klaypigeon@hotmail', 'info');

    ($time, $event, $info) = $parser->_parse_line('3340:09K;6;axis;{LLC}Andyman;12;allies;IronRabbit;bar_mp;62;MOD_RIFLE_BULLET;torso_upper');
    is($time, 3340 * 60 + 9, 'time in seconds');
    is($event, 'K', 'kill event');
    is($info, '6;axis;{LLC}Andyman;12;allies;IronRabbit;bar_mp;62;MOD_RIFLE_BULLET;torso_upper', 'kill info');
}

sub test_connect_creates_player_in_players
{
    my $parser = SOF2Logger::Parser::COD->new('cod_log.log', 0);
    
    $parser->connect_client(2101, '0;{LLC}Bunga^7');

    is($parser->game_slots->[0]->{PLAYER}->name, 
        '{LLC}Bunga^7',
        'player is created properly');
}

sub test_records_world_kills_properly
{
    # now that I know what a world kill looks like:
    # 8397:29K;3;axis;^1-^7a^1h^7X^1-^73r^1dRe^7ich;-1;world;;none;1952;MOD_EXPLOSIVE;none

    my $parser = SOF2Logger::Parser::COD->new('cod_log.log', 0);

    $parser->new_player_at_slot(3, 'AHX3RDREICH', 1, 8397 * 60 + 29);
    $parser->record_kill(
        8397 * 60 + 29, 
        '3;axis;^1-^7a^1h^7X^1-^73r^1dRe^7ich;-1;world;;none;1952;MOD_EXPLOSIVE;none');

    # until we get parsers for 1.1, 1.2 and the new formats of 1.3 and 1.4
#     is($parser->kills->deaths_for_player('AHX3RDREICH'),
#        1,
#        'deaths after a world kill');
}
