#!/usr/bin/perl -w

use strict;

use Test::More tests => 9;

use File::Basename qw/dirname/;

BEGIN { use_ok('SOF2Logger::Parser::JK2') }

my $parser = SOF2Logger::Parser::JK2->new();

isa_ok($parser, 'SOF2Logger::Parser::JK2');

ok($parser->weapon_map('MOD_SABER'), 'weapons exist');

# possibly tough line to parse
$parser->change_info_client(2600, '16 n\spec\t\2\model\{tc}phil/default\g_redteam\Empire\g_blueteam\Rebellion\c1\4\c2\5\hc\100\w\0\l\0\tt\0\tl\0');
is(SOF2Logger::Parser::SlotList->get_slot(16)->player->name, 'spec', 'change info line');

test_parse_log();

# just some basic tests to ensure that the log is parsing correctly
sub test_parse_log {
    my $parser = SOF2Logger::Parser::JK2->new(dirname(__FILE__) . '/jk2_test.log', 0);
    $parser->parse_log();

    is(SOF2Logger::Kills->kills_for_player('REDBERRY'), 33, 'kills for random player are correct');
    is(SOF2Logger::Kills->total_deaths, 234, 'total deaths are correct');

    is(SOF2Logger::Player->find('ESDARKCS')->flag_caps(), 1, 'tracking flag caps');
    is(SOF2Logger::Player->find('NOREMORSE')->flag_returns(), 2, 'flag returns');
    is(SOF2Logger::Player->find('SALTUS')->total_time, 1446, 'total time');
};
