#!/usr/bin/perl

use strict;
use warnings;

use Test::More tests => 8;

use SOF2Logger::Player;

BEGIN { use_ok('SOF2Logger::Parser::SlotList') }

my $test_player = SOF2Logger::Player->create(id => 1111, name => 'Bobby');

SOF2Logger::Parser::SlotList->put_player_in_slot(1, $test_player, 2, 100);
is(SOF2Logger::Parser::SlotList->get_slot(1)->start_time, 100, 'player time is correct');
is(SOF2Logger::Parser::SlotList->get_slot(1)->player->id, 1111, 'player id is correct');

my $billy = SOF2Logger::Player->create(id => 1234, name => 'Billy');
# try to overwrite the slot now
SOF2Logger::Parser::SlotList->put_player_in_slot(1, $billy, 1, 150);
is(SOF2Logger::Parser::SlotList->get_slot(1)->player->id, 1234, 'overwriting id is correct');
is($test_player->total_time(), 50, 'player time is correct');

SOF2Logger::Parser::SlotList->put_player_in_slot(2, $test_player, 1, 150);
my @players = SOF2Logger::Parser::SlotList->all_slots;

is(scalar @players, 2, 'num of players is correct');

SOF2Logger::Parser::SlotList->disconnect(1);
ok(!SOF2Logger::Parser::SlotList->get_slot(1));

# does not count when spectating
my $frankie = SOF2Logger::Player->create(id => 2222, name => 'Frankie');
SOF2Logger::Parser::SlotList->put_player_in_slot(1, $frankie, 2, 100, 1);
SOF2Logger::Parser::SlotList->disconnect(1, 150);
is($frankie->total_time, 0, 'frankie has not accumulated any time');



