package SOF2Logger::ParserBuilder;

use strict;
use warnings;

use SOF2Logger::Config;
use SOF2Logger::Utils qw/dprint/;

# Parser packages
use SOF2Logger::Parser::COD;
use SOF2Logger::Parser::JK2;
use SOF2Logger::Parser::SOF2::OSP;
use SOF2Logger::Parser::SOF2::ROC;
use SOF2Logger::Parser::SOF2::RPM;
use SOF2Logger::Parser::SOF2;

my $types = [
	     'JK2' => { type => 'jk2',
			mod => '.*',
			is_default => 'Y' },

	     'SOF2' => { type => 'sof2',
			 mod => '.*',
			 is_default => 'Y' },

	     'SOF2::ROC' => { type => 'sof2',
			      mod => 'roc', },

	     'SOF2::OSP' => { type => 'sof2',
			      mod => 'osp', },

	     'SOF2::RPM' => { type => 'sof2',
			      mod => 'rpm' },

	     'Q3A' => { type => 'q3a',
			mod => '.*',
			is_default => 'Y' },

	     'COD' => { type => 'Call of Duty',
			mod => '.*',
			is_default => 'Y' },

	     'COD' => { type => 'COD',
			mod => '.*',
			is_default => 'Y' },

];

sub get_parser {
  my ($class, $game_type, $game_mod) = @_;

  $game_mod ||= '';

  my $package = 'SOF2Logger::Parser::';
  my $default_parser;
  my $preferred_parser;

  my @tmp_types = @$types;

  while(@tmp_types) {
    my ($key, $val) = splice(@tmp_types, 0, 2);
    my $type = $val->{type};
    my $mod = $val->{mod};

    if ($game_type =~ /$type/i && $game_mod =~ /$mod/i) {
      $val->{is_default}
	? $default_parser = $key
	  : $preferred_parser = $key;
    }
  }

  die "Unable to determine parser for type: $game_type and mod: $game_mod"
    unless ($default_parser || $preferred_parser);

  my $parser_package = $preferred_parser ? $preferred_parser : $default_parser;
  SOF2Logger::Config->instance()->game_type($parser_package);
  $package .= $parser_package;

  dprint "Using $package";

  return $package->new(SOF2Logger::Config->instance()->log_file(),
		       SOF2Logger::Config->instance()->debug());
}

1;
