package SOF2Logger::Reports::Weapon;

use strict;
use warnings;

use Class::MethodMaker
  new_with_init => 'new',
  get_set       => [qw/kills weapon dataset config players/];
use SOF2Logger::Config;
use SOF2Logger::Kills;
use SOF2Logger::OutputHTML qw/strip_colors/;
use Template;

use constant TEMPLATE_FILE => 'IndividualPlayer.html';

sub init
{
    my ( $self, $weapon, $players ) = @_;
    $self->kills( SOF2Logger::Kills->instance );
    $self->weapon($weapon);
    $self->config( SOF2Logger::Config->instance );
    $self->players( $players );

    $self->dataset( $self->_get_dataset );
}

sub render
{
    my ( $self, $template_path ) = @_;

    my $template_config = {
        INCLUDE_PATH => $template_path,
        EVAL_PERL    => 1,
    };

    my $report_str = '';

    my $template = Template->new($template_config);
    $template->process( TEMPLATE_FILE, $self->dataset, \$report_str )
      or die $template->error();

    return $report_str;
}

sub _get_dataset
{
    my ($self)      = @_;
    my $players_ref = $self->kills->kills_by_player_for_weapon( $self->weapon );
    my $limit       = $self->config->top_weap;

    return map {
        {
            name     => $self->players->{ $_->[0] }->name,
            kills    => $_->[1],
            is_rated => !$self->players->{ $_->[0] }->is_filtered,
        }
    } @$players_ref;

}

################################################################################

=item top_weapon_handler CONFIG KILLS PLAYERS

Desc:Loop through all the weapons and determine which players have used that
weapon and how many times.

=cut

sub top_weapon_handler
{
    my ( $config, $kills, $players ) = @_;

    my $weapons = $kills->weapons;

    my $weapon_handler_html;

    foreach my $weapon_name (@$weapons)
    {

        my @players = players_with_kills_for_weapon($weapon_name);

        # ignore weapons that have no kills
        next if !@players;

      # $weapon_handler_html .= kills_by_weapon_header( $limit, $weapon_name, );

        # make all players with kills for weapon
        #		$weapon_handler_html .=
        #		  kills_by_weapon_players( $kills->kills_for_weapon($weapon_name),
        #			$limit, @players, );
        #		$weapon_handler_html .= kills_by_weapon_footer();

    }
    return $weapon_handler_html;
}

sub players_with_kills_for_weapon
{
}

1;
