package SOF2Logger::ServerStats;

# this is more of a smart data structure then it is an actual class.
# All this really needs to do is have a "check" when trying to add
# data to this "structure"... although I have no way of validating any
# of this right now.

# maybe something like an "add" method that takes a "name" and
# "value", and then I can run checks on the name and the type of value
# passed in.  Perhaps this would be something similar to Class::Meta,
# but probably not that involved.

use strict;

use Carp qw/cluck/;

use SOF2Logger::Config;

our $AUTOLOAD;

my $properties = { last_line     => '',
		   last_line_num => 0,
		   log_modtime   => 0,
		   maps          => [],
		   mods          => [],
		   server_admin  => '',
                   server_admin_email => '',
		   server_name   => '',
		   total_lines   => 0,
		   total_time    => 0 };

sub last_line {
  my ($class, $line) = @_;

  if (defined $line) {
    chomp $line;
    $properties->{last_line} = $line;
  }
  return $properties->{last_line};
}

sub game_type
{
    my ($class, $type) = @_;
    SOF2Logger::Config->instance()->game_type($type) if $type;
}

# if a method hasn't been defined, use the default autoload behaviour
sub AUTOLOAD {
  my $class = shift @_;

  my $properties_var = lc $AUTOLOAD;
  $properties_var =~ s/.*://;   # strip fully-qualified portion

  if ( exists $properties->{$properties_var} ) {
    my $prop = $properties->{$properties_var};
    if (@_) {
      if (ref $prop eq '') {
	$properties->{$properties_var} = shift @_;
      } elsif ( ref $_[0] eq 'ARRAY' ) {
	push @{$properties->{$properties_var}}, @{$_[0]};
      } else {
	push @{$properties->{$properties_var}}, @_;
      }
    }
    return $properties->{$properties_var};
  } else {
    cluck "trying to call unknown method: '$properties_var'";
  }
}

sub DESTROY {}

sub add_total_lines {
    my ($class, $lines) = @_;
    $properties->{total_lines} += $lines;
}

sub freeze {
  my ($class, $file) = @_;

  open (FILE, '>', $file)
    or die "Err: could not freeze to '$file' : $!";

  my $maps = join "\t", @{$class->maps()} if $class->maps;
  my $mods = join "\t", @{$class->mods()} if $class->mods;

  print FILE 
qq/LAST_LINE:@{[$class->last_line]}
LAST_LINE_NUM:@{[$class->last_line_num]}
LOG_MODTIME:@{[$class->log_modtime]}
SERVER_ADMIN:@{[$class->server_admin]}
SERVER_NAME:@{[$class->server_name]}
TOTAL_LINES:@{[$class->total_lines]}
TOTAL_TIME:@{[$class->total_time]}
GAME_TYPE:@{[SOF2Logger::Config->instance()->game_type]}
MAPS:$maps
MODS:$mods
/;

  close FILE;
}

sub thaw {
  my ($class, $file) = @_;

  return unless -f $file;

  open(FILE, $file)
    or die "Err: could not thaw from $file: $!\n";

  while (my $line = <FILE>) {
    chomp $line;

    my ($code, $data) = split /:/, $line, 2;

    return unless $code;

    $code = lc $code;

    $class->$code($data);
  }

  close FILE;
}

1;
