#!/usr/bin/perl -w

use strict;
use Test::More tests => 41;

use SOF2Logger::Config;
use File::Basename qw/dirname/;

eval{SOF2Logger::Config->new()};
my $config = create_test_config();

is($@, "Err: you cannot create an instance of this class\n", 'test dies when trying to create a new config');
is(ref $config, 'SOF2Logger::Config', 'isa config singleton');

is($config->help, undef, 'help is not set');

$config->parse_command_line_switches( { config_file => 'config.cfg', help => 1,  log_file => 'log_file.cfg', debug => 1 } );

is($config->config_file, 'config.cfg', 'cls config_file is set properly');
is($config->log_file, 'log_file.cfg', 'cls log_file is set properly');
is($config->debug, 1, 'cls debug is set properly');
is($config->help, 1,'cls help is set properly');

$config->parse_command_line_switches( { log_file => 'log_file.cfg'} );
# ensure that our command lines switch does not get over-riddern by the config
# file.
is($config->log_file,'log_file.cfg', 'cls log_file sticks over cfg file log_file');

test_config_file_parse();

# create a couple of filtered players
$config->{_config}{filtered_players}{HOOHAW}++;
$config->{_config}{filtered_players}{MCGRAW}++;

is($config->is_player_filtered('hoohaw'), 1, 'hoohaw is filtered');
ok(!$config->is_player_filtered('billy'), 'billy is not filtered');

ok(eq_set($config->filtered_players, ['HOOHAW', 'MCGRAW']), 'filtered players');

is($config->player_alias('foo'), 'bar', 'player is aliased correctly');
is($config->player_alias('frank'), 'jimmie', 'player is aliased correctly');
is($config->player_alias('blahblah'), 'blahblah', 'non-aliased players returns the id passed in');

ok($config->game_type ne 'test_type', 'game_type is currently NOT test_type');
SOF2Logger::Config->instance()->game_type('test_type');
is($config->game_type, 'test_type', 'game_type is set correctly');

sub create_test_config
{
    my $config = SOF2Logger::Config->instance({config_file => dirname(__FILE__) . '/logparse.conf'});
    return $config;
}

sub test_config_file_parse
{
    my %cfg = (
        admin =>  'IamGOD',
        admin_email => 'mymail@my.domain',
        server_name => 'Soldier of Fortune 2: Double Helix server',
        player_file =>  $config->full_filename('data/players.dat'),
        top_num => 100,
        top_weap => 25,
        minimum_kills => 100,
        minimum_time => 3600,
        output_file => 'player-stats.html',
        kph_file => 'player-bykph.html',
        deaths_file => 'player-bydeaths.html',
        eff_file => 'player-byeff.html',
        suicides_file => 'player-bysuicides.html',
        timeplayed_file => 'player-bytime.html',
        main_template => $config->full_filename('main-stats.html'),
        player_template =>  $config->full_filename('player-statfile.html'),
        output_dir => 'public_html',
        data_dir => $config->full_filename('data'),
        player_data_dir => 'players',
        stylesheet => 'stats.css',
        filter => $config->full_filename('player.filter'),
        player_alias_filename => $config->full_filename('test_player_aliases.txt'),
        colorize_names => 1,
        backup => 1,
        image_dir => 'ig',
    );

    while( my ($func, $return) = each %cfg) {
        is($config->$func, $return, "testing $func");
    }
}

sub usage_value
{
    return qr/NAME/;
}
