#!/usr/bin/perl

use strict;

use Test::More tests => 17;

BEGIN { use_ok 'SOF2Logger::Kills' };

my $kills = 'SOF2Logger::Kills';

# add a kill
$kills->add_kill('bob', 'billy', 'pitchfork', 'head', 1039);
my $foo = $kills->kills_for_player('bob');
is($foo, 1, 'bobs kills');

is($kills->headshots_for_player('bob'), 1, 'headshots for bob');
is($kills->headshots_for_player('billy'), 0, 'headshots for billy');

# and another one
$kills->add_kill('bob', 'hootang', 'knife', 'chest', 1041);
is($kills->kills_for_player('bob'), 2, 'bobs kills');

# one last one, to give hootang 2 deaths vs billy's 1
$kills->add_kill('bob', 'hootang', 'RL', 'stomach', 1041);

$kills->add_kill('bob', 'frank', 'RL', 'stomach', 1041);
$kills->add_kill('bob', 'frank', 'RPG', 'stomach', 1040);
$kills->add_kill('bob', 'frank', 'RPG', 'stomach', 1049);
$kills->add_kill('frankie', 'hootang', 'RL', 'chest', 1041);
$kills->add_kill('bob', 'frank', 'RL', 'stomach', 1051);
$kills->add_kill('bob', 'fred', 'RL', 'chest', 11051);
$kills->add_kill('bob', 'insanekilla', 'RPG', 'arm', 1151);
$kills->add_kill('bob', 'fruitloopZ', 'pitchfork', 'ass', 1101);

# check kills by weapon
my $ref = [
            [ 'RL'       , 4 ],
            [ 'RPG'      , 3 ],
            [ 'pitchfork', 2 ],
            [ 'knife'    , 1 ],
        ];
is_deeply($kills->kills_by_weapon_for_player('bob'), $ref, 'kills by weapon for player');

# check weapon kills
is($kills->kills_for_weapon('knife'), 1, 'kills for weapon');

# check weapon kills
is($kills->kills_for_weapon('RL'), 5, 'kills for weapon');

# check deaths
is($kills->deaths_for_player('billy'), 1, 'deaths for player');

# check tracked weapons
my $weapons = [ 'knife', 'pitchfork', 'RL', 'RPG' ];
is_deeply($kills->weapons, $weapons, 'tracked weapons');

# all kills by all weapons
$ref = [
            [ 'RL'       , 5 ],
            [ 'RPG'      , 3 ],
            [ 'pitchfork', 2 ],
            [ 'knife'    , 1 ],
        ];

is_deeply($kills->kills_by_weapon, $ref, 'all kills by all weapons');

# only kills for weapons that have kills associated with non-filtered players
$ref = [
            [ 'RL'       , 5 ],
        ];
$kills->is_filtered_player('bob', 1);
is_deeply($kills->kills_by_weapon(1), $ref, 'all kills by non-filtered players');
$kills->is_filtered_player('bob', 0);

# all kills
is($kills->total_kills, 11, 'all kills');

# all kills by player for a given weapon
$ref = [
            [ 'bob'    , 4],
            [ 'frankie', 1],
        ];
is_deeply($kills->kills_by_player_for_weapon('RL'), $ref, 'kills for weapon by player');

# most hated enemies
$ref = [
    [ 'bob', '2' ],
    [ 'frankie', '1' ]
];

is_deeply($kills->deaths_by_player_for_player('hootang'), $ref, 'deaths by player for player');

# test suicides
$kills->add_suicide('bobby', 1024);
is($kills->suicides_for_player('bobby'), 1, 'suicides for bobby');
$kills->add_suicide('billy', 2048);
$kills->add_suicide('bobby', 1560);

my $kills_by_player_for_player =  [
    [ 'frank', 4 ],
    [ 'hootang', 2 ],
    [ 'insanekilla', 1 ],
    [ 'fruitloopZ', 1 ],
    [ 'billy', 1 ],
    [ 'fred', 1 ]
];

is_deeply(
    $kills->kills_by_player_for_player('bob'), 
    $kills_by_player_for_player,
    'bobs kills by victims'
);
