#!/usr/bin/perl

use strict;
use warnings;

use Test::More tests => 4;

use SOF2Logger::Config;
use SOF2Logger::Player;
BEGIN { use_ok('SOF2Logger::OutputHTML',
	       'main_stats_for_player',
	       'kills_by_weapon_players') }

$SOF2Logger::OutputHTML::image_dir = 'images';

test_main_stats_for_player();
test_stat_line_with_bar();
test_kills_by_weapon_players();

sub test_main_stats_for_player
{
  SOF2Logger::Config->instance()->player_data_dir('players');
  my $player = SOF2Logger::Player->create(id => 'filename', name => 'billy');
  my $html = main_stats_for_player(qw/1/, $player, qw/players 25 1 0 0 0 1234 15 15/);

    my $should_be_html =
q(<tr class="cell1">
<td>1</td><td><a href="players/filename.html" class="player">billy</a></td>
<td>25</td>
<td>1</td>
<td>0</td>
<td>0</td>
<td>0</td>
<td>0h 20m 34s</td>
<td>15</td>
<td>
<img src="./images/left.gif" height="10" alt="left"><img src="./images/main.gif" height="10" width="15" alt="main"><img src="./images/right.gif" height="10" alt="right">&nbsp;15.00%
</td>
</tr>
);

    is_without_whitespace($html, $should_be_html, 'main_stats_for_player');
}

sub test_stat_line_with_bar
{
    my $html = SOF2Logger::OutputHTML::stat_line_with_bar(qw/1 foo bar/, [25, 1]);
    my $should_be_html = q(
<tr class="cell1">
  <td>1</td>
  <td>foo</td>
  <td>bar</td>
  <td>
  <img src="../images/left.gif" height="10" alt="left"><img src="../images/main.gif" height="10" width="25" alt="main"><img src="../images/right.gif" height="10" alt="right">&nbsp;25.00%
  </td>
</tr>
);

    is_without_whitespace($html, $should_be_html, 'stat line with bar');
}

sub test_kills_by_weapon_players {
  my $bobby = SOF2Logger::Player->create(id => 1234, name => 'bobby');
  my $frankie = SOF2Logger::Player->create(id => 1234, name => 'frankie');
  my $billy = SOF2Logger::Player->create(id => 1234, name => 'billy');
  my $html = kills_by_weapon_players(20,
				     20,
				     'AK-47',
				     SOF2Logger::Config->instance(),
				     ([ $bobby,  10 ],
				      [ $frankie, 6 ],
				      [ $billy,   4 ]));

  my $should_be_html = q(
<table class="dataset">
<caption>Top 20 (<strong>AK-47</strong>) - Weapon Handlers</caption>
<tr>
 <th>Players</th>
 <th>Kills</th>
 <th>Ratio</th></tr><tr class="cell1"><td>1. <a href="players/1234.html" class="player" >bobby</a></td>
 <td>10</td>
 <td>
   <img src="./images/left.gif" height="10" alt="left"><img src="./images/main.gif" height="10" width="50" alt="main"><img src="./images/right.gif" height="10" alt="right">&nbsp;50.00%
     </td></tr><tr class="cell1"><td>2. <a href="players/1234.html" class="player" >frankie</a></td>
 <td>6</td>
 <td>
   <img src="./images/left.gif" height="10" alt="left"><img src="./images/main.gif" height="10" width="30" alt="main"><img src="./images/right.gif" height="10" alt="right">&nbsp;30.00%
     </td></tr><tr class="cell2"><td>3. <a href="players/1234.html" class="player" >billy</a></td>
 <td>4</td>
 <td>
   <img src="./images/left.gif" height="10" alt="left"><img src="./images/main.gif" height="10" width="20" alt="main"><img src="./images/right.gif" height="10" alt="right">&nbsp;20.00%
     </td></tr></table>);

  is_without_whitespace($html, $should_be_html, 'kills_by_weapon_players is correct');
}

sub is_without_whitespace
{
    my ($got, $expected, $name) = @_;
    $got =~ s/\s//g;
    $expected =~ s/\s//g;

    is($got, $expected, $name)
}
