#!/usr/bin/perl -w

use strict;

use Test::More tests => 13;

BEGIN { use_ok('SOF2Logger::ServerStats') }

my $stats = 'SOF2Logger::ServerStats';

my %methods = (
    last_line => 'Shutdown game: 00000',
    last_line_num => 12345,
    log_modtime => 26280,
    server_admin => 'RiotUsSon',
    server_name => 'R.I.O.T. US!',
    total_lines => 93011,
    total_time => 100_000,
    maps => [qw/de_dust de_africa/],
    mods => [qw/rpm/], );

while (my ($key, $val) = each %methods) {
  $stats->$key($val);
  ( ref $val eq 'ARRAY' )
    ? is_deeply($stats->$key, $val, "setting $key")
      : is($stats->$key, $val, "setting $key");
}

# test serialize methods
$stats->last_line("Test line");
$stats->last_line_num(100);
$stats->log_modtime(1024);
$stats->server_admin("Henry James");
$stats->server_name("Test Server");
$stats->total_lines(2048);
$stats->total_time(5120);
$stats->maps(['DM1', 'DM6']);
$stats->mods(['CTF', 'OSP']);

my $expected = 
qq/LAST_LINE:Test line
LAST_LINE_NUM:100
LOG_MODTIME:1024
SERVER_ADMIN:Henry James
SERVER_NAME:Test Server
TOTAL_LINES:2048
TOTAL_TIME:5120
GAME_TYPE:
MAPS:de_dust	de_africa	DM1	DM6
MODS:rpm	CTF	OSP
/;

my $freeze_file = 'test_server_stats_freeze.txt';

$stats->freeze($freeze_file);

open(FILE, $freeze_file);

my @lines = <FILE>;

close FILE;

is(join('', @lines), $expected, 'seriliaze works correctly');

ok($stats->thaw('test_server_stats_freeze.txt'));

unlink('test_server_stats_freeze.txt');

$stats->add_total_lines(150);
is($stats->total_lines,2048 + 150, 'total lines read');
