package SOF2Logger::Utils;

use strict;
use base qw/Exporter/;
use List::Util qw/min/;

our @EXPORT_OK = qw/create_get_set show_debug_do show_do dprint min_defined filename_for_player strip_q3_colors/;

# returns the minimum defined value of a list
sub min_defined
{
    return min(grep { defined $_ } @_);
}

sub create_get_set
{
    my ($obj, $property) = @_;

    return if $obj->can($property);

    no strict 'refs';

    my $fq_method = $obj . "::" . $property;

    *{$fq_method} = sub {
        my $self = shift;
        my $value = shift;
        defined $value
            ? $self->{$property} = $value
    	    : return $self->{$property};
    }
}

sub get_set
{
    my ($class, $obj, $property, $value) = @_;

    defined $value
        ?  $obj->{$property} = $value
        : return $obj->{$property};
}

# prints out "$message...done" while running a sub inbetween.
# this is meant to make it easier to show what the system is
# currently doing.
sub show_do
{
    my ($message, $sub_ref) = @_;
    $| = 1;
    print "\n$message...";
    $sub_ref->();
    print "done.\n";
}

# this method is the same as show_do, but it only prints a message if
# we're running in debug mode
sub show_debug_do {
  my ($message, $sub_ref) = @_;

  SOF2Logger::Config->instance()->debug()
      ? show_do($message, $sub_ref)
        : $sub_ref->();
}


sub strip_q3_colors
{
    my ($class, $str) = @_;
    $str =~ s/\^\S//g;
    return $str;
}

sub clean_q3_color_string
{
    my ($class, $str) = @_;

    $str = $class->strip_q3_colors($str);
    return 'unknown' if !$str;

    my $str2 = $str;
    $str2 =~ s/\W//g;

    # fix names that only contain special characters
    if (!$str2) {
        # try again
        $str2 = $str;
        $str2 =~ s/\./_dot_/g;
        $str2 =~ s/\-/_dash_/g;
        $str2 =~ s/\=/_eq_/g;
        $str2 =~ s/\s/_space_/g;
        $str2 =~ s/\!/_bang_/g;
        $str2 =~ s/\@/_at_/g;
        $str2 =~ s/\#/_pound_/g;
        $str2 =~ s/\$/_dollar_/g;
        $str2 =~ s/\%/_pct_/g;
        $str2 =~ s/\^/_hat_/g;
        $str2 =~ s/\&/_amp_/g;
        $str2 =~ s/\*/_star_/g;
        $str2 =~ s/\(/_op_/g;
        $str2 =~ s/\)/_cp_/g;
        $str2 =~ s/\[/_ob_/g;
        $str2 =~ s/\]/_cb_/g;
        $str2 =~ s/\</_lt_/g;
        $str2 =~ s/\>/_gt_/g;
        $str2 =~ s/\|/_pipe_/g;
        $str2 =~ s/\?/_quest_/g;
        $str2 =~ s/(\W)//g;
        if (!$str2) {
          $str2 = 'unknown';
        }
    }
    return lc $str2;
}

sub convert_seconds_to_time_format
{
    my ($class, $seconds) = @_;
    return 0 unless $seconds;
    my $hours = int($seconds /3600);   #convert to hours
    $seconds %= 3600;                  #remove the hours we figured out
    my $minutes = int($seconds /60);
    $seconds %= 60;
    return sprintf "%3dh %02dm %02ds",$hours,$minutes,$seconds;
}

sub filename_for_player {
  my ($player) = @_;
  return $player->id . '.html';
}

sub html_pct_bar
{
    my ($class, $bar_length, $levels) = @_;
    my $config = SOF2Logger::Config->instance;
    my $image_dir = $config->image_dir;

    my $depth = $levels
        ? join '/', ('..' x $levels)
        : '.';

    my $left_img = "$depth/$image_dir/left.gif";
    my $main_img = "$depth/$image_dir/main.gif";
    my $right_img = "$depth/$image_dir/right.gif";
    my $percent = sprintf("%.2f%%", $bar_length);
    $bar_length = int $bar_length;

    return qq(
  <img src="$left_img" height="10" alt="left"><img src="$main_img" height="10" width="$bar_length" alt="main"><img src="$right_img" height="10" alt="right">&nbsp;$percent
    );
}

sub dprint
{
    my ($msg) = @_;
    return unless SOF2Logger::Config->instance->debug;
    warn "$msg\n";
}

1;

__END__
